@@ -414,13 +414,14 @@ mssinitlevel<-function(data, kmax=9, khigh=9, d="cosangle", dmat=NULL,
 	if(!is.matrix(data))
 		stop("First arg to mssinitlevel() must be a matrix")
 
-	p<-length(data[,1])
+	p<-nrow(data)
 
-        if(is.matrix(dmat))
+        if(!is.hdist(dmat)){
+           if(is.matrix(dmat) && nrow(dmat)==p && ncol(dmat)==p)
 		dmat<-as.hdist(dmat)
-
-	else
+	   else
 		dmat<-distancematrix(data,d=d)
+        }
 
 	if(dmat@Size != p)
 		stop("Data and distance matrix dimensions do not agree in mssinitlevel()")
@@ -442,7 +443,7 @@ mssinitlevel<-function(data, kmax=9, khigh=9, d="cosangle", dmat=NULL,
 		if(ord=="co")
 			medoidsord<-improveordering(medoidsdist)
 		if(ord=="clust"){
-			mpamobj<-pam(medoidsdist@Size,2,diss=TRUE)
+			mpamobj<-pam(medoidsdist@Data,2,diss=TRUE)
 			labelsmed<-mpamobj$clustering
 			medmed<-mpamobj$medoids
 			clussizes<-mpamobj$clusinfo[,1]
@@ -591,7 +592,7 @@ paircoll<-function(i,j,data,level,d="cosangle",dmat=NULL,newmed="medsil"){
 #note: this version of collap does not have silhbased arg: for use with MSS only (not silhouettes)
 collap<-function(data,level,d="cosangle",dmat=NULL,newmed="medsil"){
 	k<-level[[1]]
-	if(k<3){
+	if(k<3 && newmed!="nn"){
 		warning("Not enough medoids to use newmed='medsil' in collap() - \n using newmed='nn' instead \n") 
 		newmed<-"nn"
 	}
@@ -614,7 +615,11 @@ collap<-function(data,level,d="cosangle",dmat=NULL,newmed="medsil"){
 
 msscollap<-function(data,level,khigh,d="cosangle",dmat=NULL,newmed="medsil",within="med",between="med",impr=0){
 
-	newk<-level[[1]]
+	if(impr<0){
+          warning("impr must be positive - setting impr=0.")
+          impr<-0
+        }
+        newk<-level[[1]]
 	mss1<-labelstomss(level[[4]],dmat,khigh,within,between)
 	maxncoll<-max(0,newk-2)
         ncoll<-0
@@ -642,7 +647,10 @@ msscollap<-function(data,level,khigh,d="cosangle",dmat=NULL,newmed="medsil",with
 mssmulticollap<-function(data,level,khigh,d="cosangle",dmat=NULL,newmed="medsil",within="med",between="med",impr=0){
 	if(!is.matrix(data))
 		stop("First arg to mssmulticollap() must be a matrix")
-
+	if(impr<0){
+          warning("impr must be positive - setting impr=0.")
+          impr<-0
+        }
 	medoids<-level[[2]]
 	medoidsdata<-data[medoids,]
 	if(sum(is.na(medoidsdata))>0) 
@@ -970,16 +978,17 @@ hopach<-function(data, dmat=NULL, d="cosangle", clusters="best", K=15,
 	if(inherits(data,"ExpressionSet")) 
 		data<-exprs(data)
 	data<-as.matrix(data)
+        p<-nrow(data)
 
 	# Convert to hdist immediately #
 	if( is.null(dmat) ){
       	dmat<-distancematrix(data,d)
-	}else if( is.matrix(dmat) ){
+	}else if( is.matrix(dmat) && nrow(dmat)==p && ncol(dmat)==p){
 		dmat <- as(dmat,"hdist")
 	}else if( class(dmat) == "dist" ){
 		dmat <- hdist(Data=as.numeric(dmat), Size=attr(dmat,"Size"), Labels=(1:(attr(dmat,"Size"))), Call=as.character(attr(dmat,"call"))[3])
 	}else if(!is.hdist(dmat)){
-		stop("Distance matrix could not be created into hdist object.") 
+		stop("Distance matrix could not be transformed into hdist object.") 
 	}
 
 	if(K>15){
